'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * Created by asafdavid on 09/12/2015.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      */


var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _errio = require('errio');

var Errio = _interopRequireWildcard(_errio);

var _logger = require('../logger');

var _logger2 = _interopRequireDefault(_logger);

var _temporaryError = require('../error/temporaryError');

var _temporaryError2 = _interopRequireDefault(_temporaryError);

var _handlerResponse = require('./handlerResponse');

var _handlerResponse2 = _interopRequireDefault(_handlerResponse);

var _composePromise = require('./composePromise');

var _composePromise2 = _interopRequireDefault(_composePromise);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var MsgHandler = function () {
  function MsgHandler(name, pattern) {
    var middleware = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : { pre: [], post: [] };
    var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};

    _classCallCheck(this, MsgHandler);

    this.name = name;
    this.pattern = pattern;
    this.middleware = middleware;
    this.options = options;
  }

  /**
   * Validates if the wanted pattern match the provided handler msg
   * There are 4 options:
   * 1. Array of topics
   * 2. RegEx pattern
   * 3. Object where the keys are the topics
   * @param topic
   */


  _createClass(MsgHandler, [{
    key: 'match',
    value: function match(topic) {
      // If array, check if the topic is inside the array
      if (_lodash2.default.isArray(this.pattern)) {
        return this.pattern.indexOf(topic) > -1;
      }

      // If RegEx check the match
      if (_lodash2.default.isRegExp(this.pattern)) {
        return topic.match(this.pattern) != null;
      }

      // If object, check if the topic is part of the object
      if (_lodash2.default.isObject(this.pattern)) {
        return Object.prototype.hasOwnProperty.call(this.pattern, topic);
      }

      // Else, pattern is a string
      return this.pattern === topic;
    }

    /* eslint-disable class-methods-use-this */
    /**
     * Default msg handler - does nothing
     * @param msg
     */

  }, {
    key: 'doHandle',
    value: function doHandle(msg) {
      return _bluebird2.default.resolve(msg);
    }

    /* eslint-enable class-methods-use-this */

    /**
     * Handle the reply to a queue and ack the message
     * Handles response if it is HandlerResponse,
     * acceptable sending type (string/number/[string]/[number]) or a general object to stringify.
     * @param sender - sender object
     * @param msg - the original message to response to
     * @param res - string or handlerResponse
     * @param headers - headers to append to to response
     * @returns {*}
     */

  }, {
    key: 'reply',
    value: function reply(sender, msg, res, headers) {
      var replyPromise = _bluebird2.default.resolve();

      if (this.shouldReply(msg, res)) {
        try {
          if (!(res instanceof _handlerResponse2.default)) {
            // if acceptable type use res as body, otherwise stringify
            var body = _lodash2.default.isString(res) || _lodash2.default.isNumber(res) || _lodash2.default.isNil(res) ? res : JSON.stringify(res);
            res = new _handlerResponse2.default(body, msg, headers);
          }

          // Send a reply
          replyPromise = sender.reply(res);
          _logger2.default.instance.info('[MsgHandler] replies to a msg', {
            consumerTag: msg.fields.consumerTag,
            headers: msg.headers,
            request: msg.content,
            response: res.body
          });
        } catch (e) {
          _logger2.default.instance.error('[MsgHandler] an unexpected error has occurred while attempting to reply \n           to the msg with tag: ' + msg.fields.consumerTag + '\n           Error was ' + (e.stack || e.toString()) + '\n           response was: ' + (res.messageBody || res), { error: e, rabbitMsg: msg });

          replyPromise = _bluebird2.default.reject(e);
        }
      }

      return replyPromise.then(function () {
        return sender.ack(msg.raw);
      });
    }

    /**
     * Abstract message handler, handles acks and nacks so each handler won't have to be aware of the
     * channel. This function have two hooks: preHandle - before doHandle function, and postHandle -
     * after doHandle function
     * @param sender
     * @param msg
     * @param promise
     * @returns {Promise.<T>}
     */

  }, {
    key: 'handle',
    value: function handle(sender, msg) {
      var _this = this;

      // Bind middleware to this handler
      var pre = this.middleware.pre.map(function (func) {
        return func.bind(_this);
      });
      var post = this.middleware.post.map(function (func) {
        return func.bind(_this);
      });

      // Add pre and post middleware to the handler
      var handler = _composePromise2.default.apply(undefined, _toConsumableArray(post.reverse()).concat([this.doHandle.bind(this)], _toConsumableArray(pre)));

      // Force bluebird promise if not already
      return _bluebird2.default.resolve().then(function () {
        return handler(msg);
      }).tap(function (res) {
        return _this.reply(sender, msg, res);
      }).catch(function (err) {
        _logger2.default.instance.error('[MsgHandler] received the following error from handler - ' + _this.name, { error: err });
        // If error is NOT of the temporary type, reply as error,
        // otherwise reinsert it back to the message queue or if the TemporaryError has
        // no retries left
        if (!(err instanceof _temporaryError2.default) || msg.headers['x-attempts-left'] <= 1) {
          // if the client rejected with a non error, lets make it an error.
          err = err instanceof Error ? err : new Error(err);

          err = Errio.stringify(err);
          return _this.reply(sender, msg, err, { isError: true });
        } else if (sender.nackExchange) {
          _logger2.default.instance.error('[MsgHandler] app-nacks a msg', { error: err, consumerTag: msg.fields.consumerTag });
          sender.appNack(msg);
        } else {
          _logger2.default.instance.error('[MsgHandler] nacks a msg', { error: err, consumerTag: msg.fields.consumerTag });
          sender.nack(msg);
        }
        return _bluebird2.default.reject(err);
      });
    }

    /* eslint-disable class-methods-use-this */
    /**
     * Checks if a response should be sent back as a reply to a reply queue
     * @param originalMessage - the original message to response to
     * @param response - the response for the original message
     * @returns {boolean}
     */

  }, {
    key: 'shouldReply',
    value: function shouldReply(originalMessage, response) {
      if (response instanceof _handlerResponse2.default) {
        return !!response.replyTo;
      }
      return !!originalMessage.properties.replyTo;
    }

    /* eslint-enable class-methods-use-this */

  }]);

  return MsgHandler;
}();

exports.default = MsgHandler;