'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * Created by asafdavid on 08/12/2015.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      */


var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _handler = require('./handler');

var _handler2 = _interopRequireDefault(_handler);

var _message = require('./message');

var _message2 = _interopRequireDefault(_message);

var _logger = require('../logger');

var _logger2 = _interopRequireDefault(_logger);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var HandlerRouter = function () {
  /**
   * This class is responsible for invoking the right handler for the provided msg based on its
   * routing key.
   * @param () - use arguments variable to get all the arguments
   * @constructor
   */
  function HandlerRouter() {
    var handlers = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

    _classCallCheck(this, HandlerRouter);

    this.handlers = [];
    if (handlers) this.addRoutes(handlers);
  }

  /**
   * Add routes to the wanted router
   * @param handlers
   */


  _createClass(HandlerRouter, [{
    key: 'addRoutes',
    value: function addRoutes(handlers) {
      // Validate handlers parameter
      if (!_lodash2.default.isArray(handlers)) handlers = [handlers];

      // Initiate a new handler per type unless already received an instance of a MsgHandler
      handlers.forEach(function (handler, i) {
        // validate each handler
        if (!(handler instanceof _handler2.default)) {
          if (!_lodash2.default.isObject(handler)) throw new Error('Invalid handler', handler);
          if (!handler.name || !handler.pattern || !handler.handle) {
            throw new Error('Invalid handler, must contain name, pattern and handler');
          } else {
            var BaseHandler = handler.base ? handler.base : _handler2.default;
            var middleware = handler.middleware || undefined;
            var options = handler.options ? handler.options : {};

            var newHandlerInstance = new BaseHandler(handler.name, handler.pattern, middleware, options);
            handlers[i] = newHandlerInstance;
            newHandlerInstance.doHandle = handler.handle;
          }
        }
      });

      // Initializes data members
      this.handlers = this.handlers.concat(handlers);
    }

    /**
     * Set the default handler
     * @param defaultHandler
     */

  }, {
    key: 'setDefaultHandler',
    value: function setDefaultHandler(defaultHandler) {
      // // Validate the defaultHandler
      if (!!defaultHandler && !(defaultHandler instanceof _handler2.default)) {
        throw new Error('Invalid argument, defaultHandler must be an instance of MsgHandler');
      }

      this.defaultHandler = defaultHandler;
    }

    /* eslint-disable class-methods-use-this, no-unused-vars */
    /**
     * Function to run before the handler handle the message
     * Good for logs
     * @param sender
     * @param msg
     * @returns {Promise.<T>}
     */

  }, {
    key: 'preRoute',
    value: function preRoute(sender, msg) {
      return _bluebird2.default.resolve();
    }
    /* eslint-enable class-methods-use-this, no-unused-vars */

    /* eslint-disable class-methods-use-this, no-unused-vars */
    /**
     * Function to run after the handler handle the message
     * Good for logs
     * @param msg
     * @param res
     * @param err
     * @returns {Promise.<T>}
     */

  }, {
    key: 'postRoute',
    value: function postRoute(msg, res, err) {
      return _bluebird2.default.resolve();
    }
    /* eslint-enable class-methods-use-this, no-unused-vars */

    /**
     * Finds the handler that should handle the provided msg,
     * in case no suitable handler was found, invoke the default handler.
     * @param sender
     * @param rawMsg
     * @returns {Promise.<Message>}
     */

  }, {
    key: 'route',
    value: function route(sender, rawMsg) {
      var _this = this;

      var msg = new _message2.default(rawMsg);
      return _bluebird2.default.resolve().then(function () {
        return _this.preRoute(sender, msg);
      }).then(function () {
        var handlePromise = _bluebird2.default.resolve();

        var wasFound = _this.handlers.some(function (handler) {
          if (handler.match(msg.topic)) {
            handlePromise = handler.handle(sender, msg).then(function (res) {
              return _this.postRoute(msg, res, null);
            }).catch(function (err) {
              return _this.postRoute(msg, null, err);
            });
            return true;
          }
          return false;
        });

        if (!!_this.defaultHandler && !wasFound) {
          _logger2.default.instance.info('[HandlerRouter] no suitable was found, using default');
          handlePromise = _this.defaultHandler.handle(sender, msg).then(function () {
            return _this.postRoute(msg, null, new Error('Not Found'));
          });
        }

        return handlePromise;
      }).catch(function (e) {
        _logger2.default.instance.error('[HandlerRouter] an error has occurred during routing', { error: e });
        throw e;
      });
    }
  }]);

  return HandlerRouter;
}();

exports.default = HandlerRouter;