/**
 * Created by asafdavid on 10/12/2015.
 */
import Promise from 'bluebird';
import md5 from 'md5';
import Listener from './listener';
import Sender from './sender';
import Logger from './logger';

export default class Worker {
  constructor({ name, conn, domain, queue, router, config, options }) {
    this.name = name;
    this.conn = conn;
    this.domain = domain;
    this.queue = queue;
    this.router = router;
    this.config = config || Promise.resolve();
    this.options = options || {};

    if (this.options.logger) {
      Logger.setLogger(this.options.logger);
    }

    // Saves the requests promises this worker handles
    this.routePromise = {};
  }

  start() {
    const consumerTag = this.options.consumerTag ? `${this.options.consumerTag}-worker` : undefined;
    this.conn.connect();
    this.conn.on('open', () => {
      Logger.instance.info('[AMQP] channel is open');
      this.config().then(() => {
        this.listener = new Listener(
          this.conn.channel,
          `${this.domain}.${this.queue}`,
          consumerTag,
          this.options.qosLimit,
          this.options.pull
        );

        this.sender = new Sender(
          this.conn.channel,
          this.options.errExchange,
          this.options.nackExchange,
          this.options.timeoutExchange,
          this.options.timeoutQueueName,
          this.options.queueExpiration
        );
      }).then(() => {
        this.listener.start();
        this.listener.on('msg', (msg) => {
          // Creating unique entry to wait for promise when stopping
          const entry = `${md5(msg)}:${Date.now()}`;
          this.routePromise[entry] = this.router.route(this.sender, msg)
            .then(() => {
              delete this.routePromise[entry];
            });
        });
      });
    });
  }

  stop() {
    Logger.instance.info('[AMQP] Stopping worker...');
    return Promise.resolve()
      .then(() => this.listener.stop())
      .then(() => Promise.props(this.routePromise))
      .finally(() => this.conn.close());
  }
}
