/**
 * Created by asafdavid on 20/02/2017.
 */
import _ from 'lodash';

import BaseService from '../baseService';

/**
 * A method decorator to define a new task in a micro-service interface,
 * the decorator will add the right metadata to the
 * @param sync
 * @param description
 * @param topic
 * @param errors
 * @param baseError
 * @param timeout
 * @returns {taskDecorator}
 */
export default function Task({ sync, description, topic, errors = [], baseError, timeout }) {
  if (sync && !_.isBoolean(sync)) throw new TypeError('sync must be a boolean');
  if (description && !_.isString(description)) throw new TypeError('description must be a string');

  return function taskDecorator(target, key, descriptor) {
    // Check if a task is defined within a service
    if (!(target instanceof BaseService)) {
      throw SyntaxError('@task must be defined within a service');
    }
    if (!_.isFunction(descriptor.value)) throw SyntaxError('@task must be defined on a function');

    // Initializes the _tasks container
    if (!target._tasks) target._tasks = {};

    // Define the task metadata
    const task = {
      name: key,
      handler: descriptor.value,
      middleware: { pre: [], post: [] },
      options: {},
      sync,
      topic,
      description,
      errors,
      baseError,
      timeout,
    };

    // merge with father class
    const father = Object.getPrototypeOf(target);
    const fatherTask = father instanceof BaseService ? father._tasks[key] : undefined;
    target._tasks[key] = _.merge({}, fatherTask, task);

    const topicInTask = target._tasks[key].topic;
    if (!topicInTask || !_.isString(topicInTask)) throw new TypeError('topic is required and must be a string');
  };
}
